﻿using System;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.DirectoryServices;
using System.DirectoryServices.AccountManagement;
using DynamicwebADIntegration.Entities;

namespace DynamicwebADIntegration
{
    public class ADHelper
    {
        /// <summary>
        /// Gets all AD users
        /// </summary>
        /// <returns></returns>
        public static List<User> GetAllUsers()
        {
            List<User> ret = new List<User>();
            using (PrincipalContext context = GetPrincipalContext())
            {
                using (PrincipalSearcher searcher = new PrincipalSearcher(new UserPrincipal(context)))
                {
                    List<string> groupsToSkip = new List<string>();
                    List<string> groupsToUse = new List<string>();
                    if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings["GroupsToSkip"]))
                    {
                        groupsToSkip = ConfigurationManager.AppSettings["GroupsToSkip"].Split(new char[] { ',' }).ToList();
                    }
                    if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings["LimitToGroups"]))
                    {
                        groupsToUse = ConfigurationManager.AppSettings["LimitToGroups"].Split(new char[] { ',' }).ToList();
                    }

                    foreach (UserPrincipal adUser in searcher.FindAll())
                    {                       
                        //skip users with empty name/email/first name/last name/not enabled
                        if (!string.IsNullOrEmpty(adUser.EmailAddress) && !string.IsNullOrEmpty(adUser.Name) && adUser.Enabled.HasValue && adUser.Enabled.Value)
                        {
                            User user = new User();
                            user.Name = adUser.Name;
                            user.Email = adUser.EmailAddress;
                            user.HomeDirectory = adUser.HomeDirectory;
                            user.VoiceTelephoneNumber = adUser.VoiceTelephoneNumber;
                            user.GivenName = adUser.GivenName;
                            user.MiddleName = adUser.MiddleName;
                            user.Surname = adUser.Surname;
                            user.EmployeeId = adUser.EmployeeId;
                            user.AccountExpirationDate = adUser.AccountExpirationDate;
                            user.Enabled = adUser.Enabled;
                            //This code for getting groups works, but the exection time is icreasing a lot
                            //foreach (GroupPrincipal adUserGroup in adUser.GetGroups(context))
                            //{
                            //user.Groups.Add(new Group(adUserGroup.Name));
                            //}
                            user.Groups.AddRange(GetUserGroups(adUser, groupsToSkip, groupsToUse));
                            if (groupsToUse.Count > 0)
                            {
                                if (user.Groups.Count > 0)
                                {
                                    ret.Add(user); 
                                }
                            }
                            else
                            {
                                ret.Add(user); 
                            }                            
                        }
                        else
                        {
                            continue;
                        }                                               
                    }
                }
            }
            return ret;
        }

        private static List<Group> GetUserGroups(UserPrincipal adUser, List<string> groupsToSkip, List<string> groupsToUse)
        {
            List<Group> result = new List<Group>();
            bool checkGroupsToUse = groupsToUse.Count > 0;
            DirectoryEntry de = ((Principal)adUser).GetUnderlyingObject() as DirectoryEntry;
            if (de != null && de.Properties["memberOf"] != null && de.Properties["memberOf"].Value != null)
            {
                if (de.Properties["memberOf"].Value is Array)
                {
                    foreach (var group in (Array)de.Properties["memberOf"].Value)
                    {
                        if (group != null && !string.IsNullOrEmpty(group.ToString()))
                        {
                            string[] split = group.ToString().Split(new char[] { ',' });
                            foreach (string pair in split)
                            {
                                string[] keyValue = pair.Split('=');
                                if (keyValue.Length == 2 && keyValue[0] == "CN")
                                {
                                    bool add = false;
                                    if (checkGroupsToUse)
                                    {
                                        add = groupsToUse.Contains(keyValue[1]);                                        
                                    }
                                    else
                                    {
                                        add = (!groupsToSkip.Contains(keyValue[1]));                                        
                                    }                                    
                                    if (add)
                                    {
                                        result.Add(new Group(keyValue[1]));
                                    }
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// Gets all groups from AD where the users are present
        /// </summary>
        /// <param name="users"></param>
        /// <returns></returns>
        public static List<Group> GetGroupsFromUsers(List<User> users)
        {
            List<Group> ret = new List<Group>();
            foreach (User user in users)
            {
                foreach(Group group in user.Groups)
                {
                    if(!ret.ContainsGroup(group))
                        ret.Add(new Group(group.Name));
                }                
            }
            return ret;
        }

        /// <summary>
        /// Authenticates user in AD
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public static bool AuthenticateUser(string userName, string password)
        {
            bool ret = false;
            using (PrincipalContext pc = GetPrincipalContext())
            {
                ret = pc.ValidateCredentials(userName, password);
            }
            return ret;
        }

        private static PrincipalContext GetPrincipalContext()
        {
            PrincipalContext principalContext = null;

            string domain = ConfigurationManager.AppSettings["Domain"];
            string userName = ConfigurationManager.AppSettings["UserName"];
            string password = ConfigurationManager.AppSettings["Password"];

            if (!string.IsNullOrEmpty(domain) && !string.IsNullOrEmpty(userName) && !string.IsNullOrEmpty(password))
            {
                principalContext = new PrincipalContext(ContextType.Domain, domain, userName, password);
            }
            else
            {
                if (!string.IsNullOrEmpty(domain))
                {
                    principalContext = new PrincipalContext(ContextType.Domain, domain);
                }
                else
                {
                    principalContext = new PrincipalContext(ContextType.Domain);
                }
            }
            return principalContext;
        }
    }
}